==> LibGui-12.0.1%2B1.21.2-sources.jar: io/github/cottonmc/cotton/gui/client/Scissors.java <==
package io.github.cottonmc.cotton.gui.client;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_310;
import net.minecraft.class_332;
import org.jetbrains.annotations.Nullable;
import org.lwjgl.opengl.GL11;

import java.util.ArrayDeque;
import java.util.stream.Collectors;

/**
 * Contains a stack for GL scissors for restricting the drawn area of a widget.
 *
 * @since 2.0.0
 */
@Environment(EnvType.CLIENT)
public final class Scissors {
	private static final ArrayDeque<Frame> STACK = new ArrayDeque<>();

	private Scissors() {
	}

	/**
	 * Pushes a new scissor frame onto the stack and refreshes the scissored area.
	 *
	 * @param x the frame's X coordinate
	 * @param y the frame's Y coordinate
	 * @param width the frame's width in pixels
	 * @param height the frame's height in pixels
	 * @return the pushed frame
	 */
	public static Frame push(int x, int y, int width, int height) {
		return push(null, x, y, width, height);
	}

	/**
	 * Pushes a new scissor frame onto the stack and refreshes the scissored area.
	 *
	 * <p>If the draw context is not null, any buffered content in it will be drawn
	 * when refreshing the scissor state.
	 *
	 * @param context the associated draw context, or null if not provided
	 * @param x the frame's X coordinate
	 * @param y the frame's Y coordinate
	 * @param width the frame's width in pixels
	 * @param height the frame's height in pixels
	 * @return the pushed frame
	 */
	public static Frame push(@Nullable class_332 context, int x, int y, int width, int height) {
		Frame frame = new Frame(x, y, width, height, context);
		STACK.push(frame);
		if (context != null) context.method_51452();
		refreshScissors();

		return frame;
	}

	/**
	 * Pops the topmost scissor frame and refreshes the scissored area.
	 *
	 * @throws IllegalStateException if there are no scissor frames on the stack
	 */
	public static void pop() {
		if (STACK.isEmpty()) {
			throw new IllegalStateException("No scissors on the stack!");
		}

		var frame = STACK.pop();
		if (frame.context != null) frame.context.method_51452();
		refreshScissors();
	}

	static void refreshScissors() {
		class_310 mc = class_310.method_1551();

		if (STACK.isEmpty()) {
			// Just use the full window framebuffer as a scissor
			GL11.glScissor(0, 0, mc.method_22683().method_4489(), mc.method_22683().method_4506());
			return;
		}

		int x = Integer.MIN_VALUE;
		int y = Integer.MIN_VALUE;
		int width = -1;
		int height = -1;

		for (Frame frame : STACK) {
			if (x < frame.x) {
				x = frame.x;
			}
			if (y < frame.y) {
				y = frame.y;
			}
			if (width == -1 || x + width > frame.x + frame.width) {
				width = frame.width - (x - frame.x);
			}
			if (height == -1 || y + height > frame.y + frame.height) {
				height = frame.height - (y - frame.y);
			}
		}

		int windowHeight = mc.method_22683().method_4506();
		double scale = mc.method_22683().method_4495();
		int scaledWidth = (int) (width * scale);
		int scaledHeight = (int) (height * scale);

		// Expression for Y coordinate adapted from vini2003's Spinnery (code snippet released under WTFPL)
		GL11.glScissor((int) (x * scale), (int) (windowHeight - (y * scale) - scaledHeight), scaledWidth, scaledHeight);
	}

	/**
	 * Internal method. Throws an {@link IllegalStateException} if the scissor stack is not empty.
	 */
	static void checkStackIsEmpty() {
		if (!STACK.isEmpty()) {
			throw new IllegalStateException("Unpopped scissor frames: " + STACK.stream().map(Frame::toString).collect(Collectors.joining(", ")));
		}
	}

	/**
	 * A single scissor frame in the stack.
	 */
	public static final class Frame implements AutoCloseable {
		private final int x;
		private final int y;
		private final int width;
		private final int height;
		private final @Nullable class_332 context;

		private Frame(int x, int y, int width, int height, @Nullable class_332 context) {
			if (width < 0) throw new IllegalArgumentException("Negative width for a stack frame");
			if (height < 0) throw new IllegalArgumentException("Negative height for a stack frame");

			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			this.context = context;
		}

		/**
		 * Pops this frame from the stack.
		 *
		 * @throws IllegalStateException if: <ul>
		 *                               <li>this frame is not on the stack, or</li>
		 *                               <li>this frame is not the topmost element on the stack</li>
		 *                               </ul>
		 * @see Scissors#pop()
		 */
		@Override
		public void close() {
			if (STACK.peekLast() != this) {
				if (STACK.contains(this)) {
					throw new IllegalStateException(this + " is not on top of the stack!");
				} else {
					throw new IllegalStateException(this + " is not on the stack!");
				}
			}

			pop();
		}

		@Override
		public String toString() {
			return "Frame{ at = (" + x + ", " + y + "), size = (" + width + ", " + height + ") }";
		}
	}
}

==> LibGui-14.0.0%2B1.21.6-sources.jar: io/github/cottonmc/cotton/gui/client/Scissors.java <==
package io.github.cottonmc.cotton.gui.client;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_332;
import net.minecraft.class_8030;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayDeque;
import java.util.stream.Collectors;

/**
 * Contains a stack for GL scissors for restricting the drawn area of a widget.
 *
 * @since 2.0.0
 * @deprecated Use scissor methods in {@link class_332} instead.
 */
@Deprecated(forRemoval = true)
@Environment(EnvType.CLIENT)
public final class Scissors {
	private static final ArrayDeque<Frame> STACK = new ArrayDeque<>();

	private Scissors() {
	}

	/**
	 * Pushes a new scissor frame onto the stack and refreshes the scissored area.
	 *
	 * @param context the associated draw context
	 * @param x the frame's X coordinate
	 * @param y the frame's Y coordinate
	 * @param width the frame's width in pixels
	 * @param height the frame's height in pixels
	 * @return the pushed frame
	 */
	public static Frame push(class_332 context, int x, int y, int width, int height) {
		Frame frame = new Frame(x, y, width, height, context);
		STACK.push(frame);
		context.field_44659.method_49700(new class_8030(x, y, width, height));

		return frame;
	}

	/**
	 * Pops the topmost scissor frame and refreshes the scissored area.
	 *
	 * @throws IllegalStateException if there are no scissor frames on the stack
	 */
	public static void pop() {
		if (STACK.isEmpty()) {
			throw new IllegalStateException("No scissors on the stack!");
		}

		STACK.pop().context.method_44380();
	}

	/**
	 * Internal method. Throws an {@link IllegalStateException} if the scissor stack is not empty.
	 */
	static void checkStackIsEmpty() {
		if (!STACK.isEmpty()) {
			throw new IllegalStateException("Unpopped scissor frames: " + STACK.stream().map(Frame::toString).collect(Collectors.joining(", ")));
		}
	}

	/**
	 * A single scissor frame in the stack.
	 */
	public static final class Frame implements AutoCloseable {
		private final int x;
		private final int y;
		private final int width;
		private final int height;
		private final @Nullable class_332 context;

		private Frame(int x, int y, int width, int height, @Nullable class_332 context) {
			if (width < 0) throw new IllegalArgumentException("Negative width for a stack frame");
			if (height < 0) throw new IllegalArgumentException("Negative height for a stack frame");

			this.x = x;
			this.y = y;
			this.width = width;
			this.height = height;
			this.context = context;
		}

		/**
		 * Pops this frame from the stack.
		 *
		 * @throws IllegalStateException if: <ul>
		 *                               <li>this frame is not on the stack, or</li>
		 *                               <li>this frame is not the topmost element on the stack</li>
		 *                               </ul>
		 * @see Scissors#pop()
		 */
		@Override
		public void close() {
			if (STACK.peekLast() != this) {
				if (STACK.contains(this)) {
					throw new IllegalStateException(this + " is not on top of the stack!");
				} else {
					throw new IllegalStateException(this + " is not on the stack!");
				}
			}

			pop();
		}

		@Override
		public String toString() {
			return "Frame{ at = (" + x + ", " + y + "), size = (" + width + ", " + height + ") }";
		}
	}
}
